-- Handy shortcuts
local wv = BeneCast.WorkVars;
local BeneCastButton = BeneCast.Button;
local BeneCastPanel = BeneCast.Panel;
local BeneCastPanelDropDown = BeneCast.Panel.DropDown;
local hp = BeneCast.Helper;
local L = BeneCast.Localisation;
local const = BeneCast.Constants;

function BeneCastButton:Resize(buttonsize)

	local button, buttonborder, buttoncooldown, buttonfade, buttonrange
	-- Set Button height and width to buttonsize
	self:SetHeight(buttonsize);
	self:SetWidth(buttonsize);
	buttonborder = self.NormalTexture;
	-- Set the Button border to fit around the button
	buttonborder:SetHeight(buttonsize * 2 - (buttonsize/36) * 10);
	buttonborder:SetWidth(buttonsize * 2 - (buttonsize/36) * 10);
	buttoncooldown = self.Cooldown;
	-- Set Cooldown size
	buttoncooldown:SetScale((buttonsize / 36));
	buttoncooldown:SetPoint('CENTER', button, 'CENTER');
	buttonfade = self.Fade;
	-- Set Fade size
	buttonfade:SetHeight(buttonsize * 2 - (buttonsize/36) * 10);
	buttonfade:SetWidth(buttonsize * 2 - (buttonsize/36) * 10);
	buttonrange = self.Range;
	-- Set Range size
	buttonrange:SetHeight(buttonsize);
	buttonrange:SetWidth(buttonsize);

end

function BeneCastButton:GetButtonFromOld()

	local button;
	local len = table.getn(wv.OldButtons);
	if len > 0 then
		button = wv.OldButtons[len];
		table.remove(wv.OldButtons);
	end
	return button;
	
end

-- Make a tooltip for the BeneCastButton based on the options set
function BeneCastButton:OnEnter()

	local profileid = BeneCast:GetCurrentProfileID();

	if self.assist then
		if wv.Profiles[profileid].showtooltips then
			GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
			GameTooltip:SetText(self.assist.name);
			GameTooltip:Show();
		end
	end
	if self.focus then
		if wv.Profiles[profileid].showtooltips then
			GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
			GameTooltip:SetText(self.focus.name);
			GameTooltip:Show();
		end
	end
	if self.spell then
		if ( wv.Profiles[profileid].showtooltips and not wv.Profiles[profileid].showtooltipname ) then
			GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
			GameTooltip:SetSpell(self.spell.spellbookid, SpellBookFrame.bookType);
			GameTooltip:Show();
		elseif ( wv.Profiles[profileid].showtooltips and wv.Profiles[profileid].showtooltipname ) then
			GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
			GameTooltip:SetText(self.spell.name);
			GameTooltip:Show();
		end
	end
	if self.item then
		if self.item.bag then
			if ( wv.Profiles[profileid].showtooltips and not wv.Profiles[profileid].showtooltipname ) then
				GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
				GameTooltip:SetBagItem(self.item.bag, self.item.slot);
				GameTooltip:Show();
			elseif ( wv.Profiles[profileid].showtooltips and wv.Profiles[profileid].showtooltipname ) then
				GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
				GameTooltip:SetText(self.item.name);
				GameTooltip:Show();
			end
		else
			-- There are no more items in the inventory and the button hasn't been hidden (yet)
			GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
			GameTooltip:SetText(self.item.name);
			GameTooltip:Show();
		end
	end
	if self.macro and self.macro.name then
		if wv.Profiles[profileid].showtooltips then
			GameTooltip:SetOwner(self, 'ANCHOR_BOTTOMRIGHT');
			GameTooltip:SetText(self.macro.name);
			GameTooltip:Show();
		end
	end
	if self.reason then
		if self.reason ~= '' then
			BeneCast:Debug('Fade-reason: ' .. self.reason);
		end
	end

end

function BeneCastButton:OnLeave()
	GameTooltip:Hide();
end

function BeneCastButton:PreClick()

end

function BeneCastButton:PostClick()

end

function BeneCastButton:SPELL_UPDATE_COOLDOWN()
	self:UpdateCooldown();
end

function BeneCastButton:OnShow()
	self:UpdateCooldown();
end

-- Update the cooldown for a button on events
function BeneCastButton:UpdateCooldown()

	-- If the unit for this button does not exist return
	if not UnitExists(self.unit) then
		return;
	end
	
	-- No need to update for a hidden button
	if ( not self:IsVisible() ) then
		return;
	end
	
	if self.spell then
		-- Funky Cooldown stuff that I'm not quite sure how exactly it all works
		local start, duration, enable = GetSpellCooldown(self.spell.name);
		CooldownFrame_SetTimer(self.Cooldown, start, duration, enable);
		if ( enable == 1 ) then
			self.Icon:SetVertexColor(1.0, 1.0, 1.0);
		else
			self.Icon:SetVertexColor(0.4, 0.4, 0.4);
		end
	end
	if self.item then
		local start, duration, enable = GetSpellCooldown(self.item.name);
		CooldownFrame_SetTimer(self.Cooldown, start, duration, enable);
		if ( enable == 1 ) then
			self.Icon:SetVertexColor(1.0, 1.0, 1.0);
		else
			self.Icon:SetVertexColor(0.4, 0.4, 0.4);
		end
	end

end

-- Register events for BeneCastButtons
function BeneCastButton:OnLoad()

	self:RegisterEvent('SPELL_UPDATE_COOLDOWN');

end

function BeneCastButton:SetFade(showbutton, reason)
	self.reason = reason;
	if showbutton then
		self.Fade:Hide();
	else
		self.Fade:Show();
	end
end

function BeneCastButton:NewButton(panel, prevbutton)

	if InCombatLockdown() then
		return;
	end
	local seqno = prevbutton.seqno;
	if seqno == nil then
		seqno = 1;
	else
		seqno = seqno + 1;
	end
	local buttonname = 'BeneCastButton_' .. pfid .. '_' .. unit .. '_' .. seqno;
	local button = self:GetButtonFromOld();
	if not button then
		button = CreateFrame('Button', buttonname , panel, 'BeneCastButtonTemplate');
		button.NormalTexture = getglobal(buttonname .. 'NormalTexture');
		button.Cooldown = getglobal(buttonname .. 'Cooldown');
		button.Fade = getglobal(buttonname .. 'Fade');
		button.Range = getglobal(buttonname .. 'Range');
		button.Duration = getglobal(buttonname .. 'Duration');
		hp._CopyTableToExisting(BeneCastButton,button,true);
		button:SetFrameStrata('LOW');
		button:SetScript('PostClick',self.PostClick);
		button:SetScript('PreClick',self.PreClick);
		button:SetScript('OnEnter',self.OnEnter);
		button:SetScript('OnEvent',function(event, ...)
			if self[event] then
				self[event](...);
			end
		end);
		button:SetScript('OnShow',self.OnShow);
		button:SetScript('OnLeave',self.OnLeave);
		button:SetScript('OnLoad',self.OnLoad);
	else
		button:SetName(buttonname);
		button.NormalTexture:SetName(buttonname .. 'NormalTexture');
		button.Cooldown:SetName(buttonname .. 'Cooldown');
		button.Fade:SetName(buttonname .. 'Fade');
		button.Range:SetName(buttonname .. 'Range');
		button.Duration:SetName(buttonname .. 'Duration');
		button:SetParent(panel);
		button:SetFrameLevel(panel:GetFrameLevel() + 1);
	end
	button:OnLoad();
	button.pf = panel.pf;
	button.unit = panel.unit;
	button.seqno = seqno;
	button.panel = panel;
	button.previous = prevbutton;
	button:SetAttribute('unit',button.unit);
	button:Hide();
	table.insert(panel.buttons,button);
	if prevbutton.next then
		button.next = prevbutton.next;
		prevbutton.next = button;
		panel:ScanButtons();
	else
		prevbutton.next = button;
	end
		
	return button;
end

-- Function to hide buttons if they're not to be shown
function BeneCastButton:Retire()

	if InCombatLockdown() then
		return;
	end

	if self.Fade:IsShown() then
		self.Fade:Hide();
	end
	if self.Range:IsShown() then
		self.Range:Hide();
	end
	if self:IsShown() then
		self:Hide();
	end
	self:ClearAction();
	self.unit = nil;
	self.pf = nil;
	self.seqno = nil;
	
	self.previous.next = self.next;
	self.next = nil;
	self.previous = nil;
	
	if self.panel then
		self.panel:ScanButtons();
		self.panel:OrientButtons();
		self.panel = nil;
	end
	button:ClearAllPoints();
	button:SetParent(UIParent);
	button:UnRegisterAllEvents();

end

function BeneCastButton:ClearAction()
	self.spell = nil;
	self.item = nil;
	self.assist = nil;
	self.focus = nil;
	self.macro = nil;
	hp._SetManyAttributes(self,'type',nil,'spell',nil,'item',nil,'macrotext',nil);
end

function BeneCastButton:SetAction(action)
	self:ClearAction();
	if action.spell then
		self.spell = action;
		hp._SetManyAttributes(self,'type','spell','spell',action.name);
	elseif action.item then
		self.item = action;
		hp._SetManyAttributes(self,'type','item','item',action.name);
	elseif action.assist then
		self.assist = action;
		hp._SetManyAttributes(self,'type','assist');
	elseif action.focus then
		self.focus = action;
		hp._SetManyAttributes(self,'type','focus');
	elseif action.macro then
		self.macro = action;
		hp._SetManyAttributes(self,'type','macro','macrotext',action.text);
	end
	self.Icon:SetTexture(action.icon);
end