﻿-- Handy shortcuts
local wv = BeneCast.WorkVars;
local BeneCastButton = BeneCast.Button;
local BeneCastPanel = BeneCast.Panel;
local BeneCastPanelDropDown = BeneCast.Panel.DropDown;
local hp = BeneCast.Helper;
local L = BeneCast.Localisation;
local const = BeneCast.Constants;

BeneCast.GetSnapToOption = function()

	BeneCast:Debug('BeneCast.GetSnapToOption()');
	if wv.Setup.SnapTo == nil then
		wv.Setup.SnapTo = 'BeneCast';
	end
	return BeneCast.SnapTo[wv.Setup.SnapTo].name;
	
end

BeneCast.SetSnapToOption = function(addon_name)

	BeneCast:Debug('BeneCast.SetSnapToOption()');
	for addonname, addon in pairs(wv.SnapToFound) do
		if addon.name == addon_name then
			wv.Setup.SnapTo = addonname;
			BeneCast.SnapTo:ApplyConfigTable();
			BeneCast.SnapTo:SnapPanels();
			do break end;
		end
	end
	
end

BeneCast.GenerateSnapToOptions = function()

	BeneCast:Debug('BeneCast.GenerateSnapToOptions()');
	local snaptooptions = hp.new();
	for addonname, addon in pairs(wv.SnapToFound) do
		tinsert(snaptooptions,addon.name);
	end
	return snaptooptions;
	
end

BeneCast.GetSetupOption = function(info)

	return wv.Setup[info[#info]];
	
end

BeneCast.SetSetupOption = function(info,value)

	local varname;
	varname = info[#info];
	wv.Setup[varname]=value;
	BeneCast:Print(varname .. " was set to: " .. tostring(value) );
	if varname == 'hideminimapbutton' then
		if MinimapButton then
			if value then
				MinimapButton:Hide();
			else
				MinimapButton:Show();
			end
		end
	end

end

BeneCast.GetProfileSettingsOption = function(info)

	return wv.Profiles[info[#info-2]][info[#info]];
	
end

BeneCast.SetProfileSettingsOption = function(info, value)

	local pfid, option;
	pfid = info[1];
	option = info[#info];
	wv.Profiles[pfid][option] = value;
	if option == 'buttonsize' then
		BeneCast:ResizeButtonsForProfile(pfid);
	
	elseif option == 'flashasfade' or option == 'showallbuttons' then
		BeneCast:UpdatePanelsForProfile(pfid);
		
	elseif option == 'checkrange' then
		if value then
			BeneCast.Timers.CheckRange:Enable();
		else
			BeneCast.Timers.CheckRange:Disable();
		end
	
	elseif option == 'showpets' then
		-- Attach the panels again, just to be sure
		BeneCast.SnapTo:SnapPanels();
		
		for pet, owner in pairs(const.OwnersByPet) do
			BeneCast:UpdatePanelsForUnit(pet);
		end
		
	elseif option == 'showtargettarget' then
		-- Attach the panels again, just to be sure
		BeneCast.SnapTo:SnapPanels();
		
		-- Redraw the buttons for the target of target
		BeneCast:UpdatePanelsForUnit('targettarget');
	
	end
		
	BeneCast:Print(info[#info] .. " was set to: " .. tostring(value) );

end

BeneCast.ToggleConfigWindow = function()

	LibStub("AceConfigDialog-3.0"):Open('BeneCast');

end

BeneCast.GetActionSetting = function(info)
	local pfid = info[1];
	local configname = info[2];
	local config = wv.Profiles[pfid].config[configname];
	if config[info.arg] == nil then
		config[info.arg] = { enabled = false };
	end
	local setting = config[info.arg];
	return setting.enabled;
end

BeneCast.SetActionSetting = function(info, val)
	local pfid = info[1];
	local configname = info[2];
	local mini, maxi;
	if wv.Profiles[pfid].cascadeplayer and configname == 'player' then
		mini = 1;
		maxi = table.getn(const.Configs);
	else
		mini = hp._GetKeyForVal(const.Configs, configname);
		maxi = mini;
	end
	for i = mini, maxi do
		configname = const.Configs[i];
		local config = wv.Profiles[pfid].config[configname];
		if config[info.arg] == nil then
			config[info.arg] = { enabled = false };
		end
		local setting = config[info.arg];
		setting.enabled = val;
	end		
	BeneCast:UpdatePanelsForProfile(pfid);
end

wv.DefaultOptions = {
    name = 'benecast',
    desc = const.Name .. ' ' .. const.Version,
    type = 'group',
    args = {
		win = {
			type = 'execute',
			name = 'win',
			desc = L['Settings'],
			guiHidden = true, -- The GUI doesn't need to show this
			func = BeneCast.ToggleConfigWindow,
			order = 1,
		},
		general = {
			type = 'group',
			name = 'general',
			desc = L['General'],
			order = 2,
			args = {
				snapto = {
					type = 'select',
					name = 'snapto',
					desc = L['Snap To Options'],
					values = BeneCast.GenerateSnapToOptions,
					get = BeneCast.GetSnapToOption,
					set = BeneCast.SetSnapToOption,
					order = 1,
				},
				debug = {
					type = 'toggle',
					name = 'debug',
					desc = 'Debug',
					get = BeneCast.GetSetupOption,
					set = BeneCast.SetSetupOption,
					order = 2,
				},
				hideminimapbutton = {
					type = 'toggle',
					name = 'hideminimapbutton',
					desc = L['Hide the minimap button'],
					get = BeneCast.GetSetupOption,
					set = BeneCast.SetSetupOption,
					order = 3,
				},
			},
		},
        ooc = {
			type = 'group',
			name = 'ooc',
			desc = L['Out of Combat'],
			order = 3,
			args = {				
				settings = {
					type = 'group',
					name = 'settings',
					desc = L['Settings'],
					order = 1,
					args = {
						orientation = {
							type = 'select',
							name = 'orientation',
							desc = L['Orientation'],
							values = function(info)
								local vals = hp.new();
								local currentorient = 'RIGHT';
								local orientobj;
								while table.getn(vals) == 0 and currentorient ~= 'RIGHT' do
									orientobj = const.Orientations[currentorient];
									tinsert(vals,orientobj.translation);
									currentorient = orientobj.next;
								end
								return vals;
							end,
							get = function(info)
								return const.Orientations[wv.Profiles[info[1]].orientation].translation;
							end,
							set = function(info,val)
								wv.Profiles[info[1]].orientation = hp._GetKeyForValInMember(const.Orientations,'translation',val);
							end,
							order = 1,
						},
						buttonsize = {
							type = 'range',
							name = 'buttonsize',
							desc = L['Button Size'],
							min = 8,
							max = 36,
							step = 1,
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 2,
						},
						flashasfade = {
							type = 'toggle',
							name = 'flashasfade',
							desc = L['Flash as Fade'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 3,
						},
						checkrange = {
							type = 'toggle',
							name = 'checkrange',
							desc = L['Check Range for Target'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 4,
						},
						showtooltips = {
							type = 'toggle',
							name = 'showtooltips',
							desc = L['Show Button Tooltips'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 5,
						},
						showtooltipname = {
							type = 'toggle',
							name = 'showtooltipname',
							desc = L['Spell Name Only'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 6,
						},
						showallbuttons = {
							type = 'toggle',
							name = 'showallbuttons',
							desc = L['Always Show Buttons'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 7,
						},
						unlockbuttons = {
							type = 'toggle',
							name = 'unlockbuttons',
							desc = L['Unlock Buttons'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 8,
						},
						cascadeplayer = {
							type = 'toggle',
							name = 'cascadeplayer',
							desc = L['Cascade Player Config'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 9,
						},
						showpets = {
							type = 'toggle',
							name = 'showpets',
							desc = L['Show Pet Buttons'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 10,
						},
						showtargettarget = {
							type = 'toggle',
							name = 'showtargettarget',
							desc = L["Show Target's Target"],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
							order = 11,
						},
					},
				},
				player = {
					type = 'group',
					name = L['Player'],
					order = 2,
					args = { }, -- This will be filled as the spells load
				},
				pet = {
					type = 'group',
					name = L['Pet'],
					order = 3,
					args = { }, -- This will be filled as the spells load
				},
				party = {
					type = 'group',
					name = L['Group Members'],
					order = 4,
					args = { }, -- This will be filled as the spells load
				},
				partypet = {
					type = 'group',
					name = L['Group Pets'],
					order = 5,
					args = { }, -- This will be filled as the spells load
				},
				other = {
					type = 'group',
					name = L['Non-Group Members'],
					order = 6,
					args = { }, -- This will be filled as the spells load
				},
			},
        },
    },
};

wv.DefaultOptions.args.combat = hp._CopyTable(wv.DefaultOptions.args.ooc, true);
wv.DefaultOptions.args.combat.name = 'combat';
wv.DefaultOptions.args.combat.desc = L['In Combat'];
wv.DefaultOptions.args.combat.order = wv.DefaultOptions.args.combat.order + 1;

-- Function to generate an optiontable from a single spell/item/...-name and a config
-- For now this is a toggle, but later it can be anything
function BeneCast:ActionToConfig(shortgroupname, fullgroupname)

	return {
		type = 'toggle',
		name = shortgroupname,
		arg = fullgroupname,
		get = BeneCast.GetActionSetting,
		set = BeneCast.SetActionSetting,
	};

end

-- Function to generate entire spellconfig by the known spells-list
function BeneCast:GenerateSpellConfigFor(configtable)

	local propergrp;
	
	-- First add assist and focus
	if configname ~= 'player' then
		configtable['assist'] = self:ActionToConfig(BeneCast.Assist.group, BeneCast.Assist.group);
	end
	configtable['focus'] = self:ActionToConfig(BeneCast.Focus.group, BeneCast.Focus.group);
	
	-- Then do spells
	for group, spell in pairs(wv.Spells) do
		if spell.target and const.TargetTypes[spell.target][configname] then
			propergrp = string.gsub(group, ' ', '');
			configtable[propergrp] = self:ActionToConfig(propergrp, group);
		end
	end
	
	-- TODO: Items and macro's once the parsing of that is done

end

-- Function to do the above for ALL configs
function BeneCast:GenerateAllSpellConfigs(optiontable)

	local configtable;
	for pfid, profile in pairs(wv.Profiles) do
		for i, configname in pairs(const.Configs) do
			configtable = optiontable.args[pfid].args[configname].args;
			self:GenerateSpellConfigFor(configtable);
		end
	end

end